﻿using System.Collections;
using System.Collections.Generic;
using UnityEngine;

[ExecuteInEditMode]
[RequireComponent(typeof(Camera))]
public class ContoursDoG : PostEffectsBase
{
    [Range(0, 5.0f)]
    public float sigma1 = 1.0f;
    [Range(0, 5.0f)]
    public float sigma2 = 1.0f;
    [Range(0, 0.2f)]
    public float DoGThreshold = 0.01f;
    [Range(0, 0.2f)]
    public float diffThreshold = 0.01f;
    [Range(0, 0.5f)]
    public float DoGFalloff = 0.01f;
    [Range(0, 0.5f)]
    public float diffFalloff = 0.01f;
    void Start()
    {
        shader = Shader.Find("NPR/ContoursDoG");
        CreateMaterial();
    }

    void OnRenderImage(RenderTexture source, RenderTexture destination)
    {
        if (CreateMaterial())
        {
            RenderTexture lum = RenderTexture.GetTemporary(source.width, source.height, 0, RenderTextureFormat.DefaultHDR);
            Graphics.Blit(source, lum, material, 0);
            RenderTexture G1 = RenderTexture.GetTemporary(source.width, source.height, 0, RenderTextureFormat.DefaultHDR);
            material.SetFloat("sigma", sigma1);
            Graphics.Blit(lum, G1, material, 1);
            RenderTexture G2 = RenderTexture.GetTemporary(source.width, source.height, 0, RenderTextureFormat.DefaultHDR);
            material.SetFloat("sigma", sigma2);
            Graphics.Blit(lum, G2, material, 1);
            material.SetTexture("Gauss1Tex", G1);
            material.SetTexture("Gauss2Tex", G2);
            material.SetFloat("DoGThreshold", DoGThreshold);
            material.SetFloat("diffThreshold", diffThreshold);
            material.SetFloat("DoGFalloff", DoGFalloff);
            material.SetFloat("diffFalloff", diffFalloff);
            Graphics.Blit(source, destination, material, 2);
            RenderTexture.ReleaseTemporary(lum);
            RenderTexture.ReleaseTemporary(G1);
            RenderTexture.ReleaseTemporary(G2);
        }
        else
        {
            Graphics.Blit(source, destination);
        }
    }
}
